@@ -138,7 +138,7 @@ DataImpl :: GroupHeaders :: build_references_header (const Article* article, std
   setme.clear ();
   const Quark& message_id (article->message_id);
   const ArticleNode * node (find_node (message_id));
-  while (node->_parent != 0) {
+  while (node && node->_parent) {
     node = node->_parent;
     const StringView& ancestor_mid = node->_mid.to_view ();
     setme.insert (0, ancestor_mid.str, ancestor_mid.len);
@@ -190,9 +190,8 @@ DataImpl :: unref_group   (const Quark& group)
   //std::cerr << LINE_ID << " group " << group << " refcount down to " << h->_ref << std::endl;
   if (h->_ref == 0)
   {
-    if (h->_dirty) {
+    if (h->_dirty)
       save_headers (*_data_io, group);
-    }
     h->_dirty = false;
     free_group_headers_memory (group);
   }
@@ -522,7 +521,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
         }
         targets_v.resize (target_it - &targets_v.front());
         targets.sort();
-        const bool expired (targets.empty());
+        bool expired (targets.empty());
         a.xref.swap (targets);
 
         // is_binary [total_part_count found_part_count]
@@ -550,6 +549,10 @@ DataImpl :: load_headers (const DataIO   & data_io,
             s.ltrim ();
             s.pop_token (tok);
             const unsigned long number (view_to_ul (tok));
+            if (!a.has_part(number)) { // corrupted entry
+              expired = true;
+              break;
+            }
             Article::Part& p (a.get_part (number));
             s.ltrim ();
             s.pop_token (tok);
@@ -602,7 +605,7 @@ DataImpl :: load_headers (const DataIO   & data_io,
 
 namespace
 {
-  typedef std::map < pan::Quark, char > quark_to_symbol_t;
+  typedef Loki::AssocVector < pan::Quark, char > quark_to_symbol_t;
 
   struct QuarkToSymbol {
     ~QuarkToSymbol () {}
@@ -619,7 +622,7 @@ namespace
     }
   };
 
-  typedef std::map<Quark,unsigned long> frequency_t;
+  typedef Loki::AssocVector<Quark,unsigned long> frequency_t;
 
   const char * lookup_symbols ("abcdefghijklmnopqrstuvwxyz"
                                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -721,7 +724,6 @@ DataImpl :: save_headers (DataIO                       & data_io,
     *out << "2\t # file format version number\n";
 
     // xref lookup section
-    typedef std::map<Quark,unsigned long> frequency_t;
     frequency_t frequency;
     foreach_const (std::vector<Article*>, articles, ait)
       foreach_const (Xref, (*ait)->xref, xit)
@@ -739,6 +741,7 @@ DataImpl :: save_headers (DataIO                       & data_io,
     // header section
     *out << articles.size() << endl;
     std::string references;
+    std::string part_mid;
     foreach_const (std::vector<Article*>, articles, ait)
     {
       ++article_count;
@@ -783,11 +786,11 @@ DataImpl :: save_headers (DataIO                       & data_io,
           ++part_count;
           const Article::Part& p (*pit);
           *out << '\t' << number << ' ';
-          const std::string tmp (p.get_message_id (message_id));
-          if (message_id.to_view() == tmp)
+          p.get_message_id (message_id, part_mid);
+          if (message_id.to_view() == part_mid)
             *out << '"';
           else
-            *out << tmp;
+            *out << part_mid;
           *out << ' ' << p.bytes << '\n';
            first = false;
         }
@@ -805,6 +808,9 @@ DataImpl :: save_headers (DataIO                       & data_io,
 void
 DataImpl :: save_headers (DataIO& data_io, const Quark& group) const
 {
+  if (_unit_test)
+    return;
+
    pan_return_if_fail (!group.empty());
 
    TimeElapsed timer;
@@ -902,6 +908,25 @@ DataImpl :: get_article_scores (const Quark         & group,
   _article_filter.get_article_scores (*this, sections, group, article, setme);
 }
 
+void
+DataImpl :: rescore_articles (const Quark& group, const quarks_t mids)
+{
+  GroupHeaders * gh (get_group_headers (group));
+  if (!gh) // group isn't loaded
+    return;
+
+  ArticleFilter::sections_t sections;
+  _scorefile.get_matching_sections (group.to_view(), sections);
+  nodes_v nodes;
+  find_nodes (mids, gh->_nodes, nodes);
+  foreach (nodes_v, nodes, it) {
+    if ((*it)->_article) {
+      Article& a (*(*it)->_article);
+      a.score = _article_filter.score_article (*this, sections, group, a);
+    }
+  }
+}
+
 void
 DataImpl :: rescore ()
 {
@@ -1088,20 +1113,7 @@ DataImpl :: on_articles_removed (const quarks_t& mids) const
 void
 DataImpl :: on_articles_changed (const Quark& group, const quarks_t& mids, bool do_refilter)
 {
-  // if the articles are loaded, rescore them...
-  GroupHeaders * gh (get_group_headers (group));
-  if (gh) {
-    ArticleFilter::sections_t sections;
-    _scorefile.get_matching_sections (group.to_view(), sections);
-    nodes_v nodes;
-    find_nodes (mids, gh->_nodes, nodes);
-    foreach (nodes_v, nodes, it) {
-      if ((*it)->_article) {
-        Article& a (*(*it)->_article);
-        a.score = _article_filter.score_article (*this, sections, group, a);
-      }
-    }
-  }
+  rescore_articles (group, mids);
 
   // notify the trees that the articles have changed...
   foreach (std::set<MyTree*>, _trees, it)
@@ -1116,6 +1128,8 @@ DataImpl :: on_articles_added (const Quark& group, const quarks_t& mids)
     Log::add_info_va (_("Added %lu articles to %s."),
                       mids.size(), group.c_str());
 
+    rescore_articles (group, mids);
+
     foreach (std::set<MyTree*>, _trees, it) {
       debug ("This tree has a group " << (*it)->_group);
       if ((*it)->_group == group) {
