@@ -27,9 +27,9 @@
 
 #include <pan/general/quark.h>
 #include <pan/general/string-view.h>
+#include <pan/usenet-utils/scorefile.h>
 #include <pan/data/article.h>
 #include <pan/data/article-cache.h>
-#include <pan/data/scorefile.h>
 #include <pan/data/server-info.h>
 
 namespace pan
@@ -440,10 +440,12 @@ namespace pan
 
           /** the quirky way of incrementing 'it' is to prevent it from being
               invalidated if on_tree_change() calls remove_listener() */
-          void fire_diffs (const Diffs& diffs) const {
-            listeners_t::iterator it, e;
-            for (it=_listeners.begin(), e=_listeners.end(); it!=e; )
-              (*it++)->on_tree_change (diffs);
+          void fire_diffs (const Diffs& d) const {
+            if (!d.added.empty() || !d.removed.empty() || !d.reparented.empty() || !d.changed.empty()) {
+              listeners_t::iterator it, e;
+              for (it=_listeners.begin(), e=_listeners.end(); it!=e; )
+                (*it++)->on_tree_change (d);
+            }
           }
               
 
