@@ -33,10 +33,10 @@ extern "C" {
 #include <pan/general/file-util.h>
 #include <pan/general/foreach.h>
 #include <pan/general/log.h>
+#include <pan/general/utf8-utils.h>
 #include <pan/usenet-utils/gnksa.h>
 #include <pan/usenet-utils/message-check.h>
 #include <pan/usenet-utils/mime-utils.h>
-#include <pan/usenet-utils/utf8-utils.h>
 #include <pan/data/data.h>
 #include <pan/tasks/task-post.h>
 #include "pad.h"
@@ -194,7 +194,7 @@ namespace
     { "file-menu", NULL, N_("_File") },
     { "edit-menu", NULL, N_("_Edit") },
     { "profile-menu", NULL, N_("_Profile") },
-    { "charsets-menu", NULL, N_("Character Encoding") },
+    { "charsets-menu", NULL, N_("Set Character Encoding") },
     { "editors-menu", NULL, N_("Set Editor") },
     { "post-toolbar", NULL, "post" },
     { "post-article", GTK_STOCK_EXECUTE, N_("_Send Article"), "<control>Return", N_("Send Article Now"), G_CALLBACK(do_send) },
@@ -445,7 +445,7 @@ PostUI :: manage_editors ()
 
   char b[512], lab[512];
   g_snprintf (b, sizeof(b), _("Editors"));
-  g_snprintf (lab, sizeof(lab), "        <b>%s:</b>        ", b);
+  g_snprintf (lab, sizeof(lab), "\t\t\t<b>%s:</b>\t\t\t", b);
   GtkWidget * w = gtk_label_new (NULL);
   gtk_label_set_markup (GTK_LABEL(w), lab);
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, 0);
@@ -470,7 +470,9 @@ PostUI :: manage_editors ()
 
   gtk_dialog_run (GTK_DIALOG (d));
 
-  const std::string text (get_body ());
+  GtkTextIter start, end;
+  gtk_text_buffer_get_bounds (buf, &start, &end);
+  char * text = gtk_text_buffer_get_text (buf, &start, &end, false);
   commands.clear ();
   StringView token, v (text);
   while (v.pop_token (token, '\n')) {
@@ -478,6 +480,7 @@ PostUI :: manage_editors ()
     if (!token.empty())
       commands.push_back (token.to_string());
   }
+  g_free (text);
 
   _profiles.set_editors (commands);
   add_editor_list ();
@@ -973,22 +976,12 @@ PostUI :: spawn_editor ()
   }
 
   // read the file contents back in
-  if (ok) {
-    GError * err (0);
-    char * body (0);
-    gsize body_len (0);
-    g_file_get_contents (fname, &body, &body_len, &err);
-    if (err != NULL) {
-      Log::add_err_va (_("Error reading file \"%s\": %s"), err->message, g_strerror(errno));
-      g_clear_error (&err);
-      ok = false;
-    } else {
-      GtkTextIter start, end;
-      gtk_text_buffer_get_bounds (buf, &start, &end);
-      gtk_text_buffer_delete (buf, &start, &end);
-      gtk_text_buffer_insert (buf, &start, body, body_len);
-    }
-    g_free (body);
+  std::string txt;
+  if (ok && file :: get_text_file_contents (fname, txt)) {
+    GtkTextIter start, end;
+    gtk_text_buffer_get_bounds (buf, &start, &end);
+    gtk_text_buffer_delete (buf, &start, &end);
+    gtk_text_buffer_insert (buf, &start, txt.c_str(), txt.size());
   }
 
   // cleanup
@@ -1019,25 +1012,29 @@ namespace
 void
 PostUI :: open_draft ()
 {
-  std::string& draft_filename (get_draft_filename ());
-
   GtkWidget * d = gtk_file_chooser_dialog_new (_("Open Draft Article"),
                                                GTK_WINDOW(_root),
                                                GTK_FILE_CHOOSER_ACTION_OPEN,
                                                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                                NULL);
-  if (!gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(d), draft_filename.c_str()))
+
+  std::string& draft_filename (get_draft_filename ());
+  if (g_file_test (draft_filename.c_str(), G_FILE_TEST_IS_DIR))
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(d), draft_filename.c_str());
+  else
     gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(d), draft_filename.c_str());
+
   if (gtk_dialog_run(GTK_DIALOG(d)) == GTK_RESPONSE_ACCEPT)
   {
     char * pch = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(d));
     draft_filename = pch;
     g_free (pch);
 
-    if (g_file_get_contents (draft_filename.c_str(), &pch, NULL, NULL))
+    std::string txt;
+    if (file :: get_text_file_contents (draft_filename, txt))
     {
-      GMimeStream * stream = g_mime_stream_mem_new_with_buffer (pch, strlen(pch));
+      GMimeStream * stream = g_mime_stream_mem_new_with_buffer (txt.c_str(), txt.size());
       GMimeParser * parser = g_mime_parser_new_with_stream (stream);
       GMimeMessage * message = g_mime_parser_construct_message (parser);
       if (message) {
@@ -1046,7 +1043,6 @@ PostUI :: open_draft ()
       }
       g_object_unref (G_OBJECT(parser));
       g_object_unref (G_OBJECT(stream));
-      g_free (pch);
     }
 
   }
@@ -1083,6 +1079,17 @@ namespace
         && (name.strstr ("X-Draft-") != name.str);
     return keep;
   }
+
+  /**
+   * Works around a GMime bug that uses `Message-Id' rather than `Message-ID'
+   */
+  void pan_g_mime_message_set_message_id (GMimeMessage *msg, const char *mid)
+  {
+    g_mime_message_add_header (msg, "Message-ID", mid);
+    char * bracketed = g_strdup_printf ("<%s>", mid);
+    g_mime_header_set (GMIME_OBJECT(msg)->headers, "Message-ID", bracketed);
+    g_free (bracketed);
+  }
 }
 
 GMimeMessage*
@@ -1149,11 +1156,11 @@ PostUI :: new_message_from_ui (Mode mode)
     g_mime_message_set_header (msg, "User-Agent", get_user_agent());
 
   // Message-ID
-  if (mode==POSTING && _prefs.get_flag (MESSAGE_ID_PREFS_KEY, true)) {
+  if (mode==POSTING && _prefs.get_flag (MESSAGE_ID_PREFS_KEY, false)) {
     const std::string message_id = !profile.fqdn.empty()
       ? GNKSA::generate_message_id (profile.fqdn)
       : GNKSA::generate_message_id_from_email_address (profile.address);
-    g_mime_message_set_message_id (msg, message_id.c_str());
+    pan_g_mime_message_set_message_id (msg, message_id.c_str());
   }
 
   // body & charset
@@ -1188,8 +1195,6 @@ PostUI :: new_message_from_ui (Mode mode)
 void
 PostUI :: save_draft ()
 {
-  std::string& draft_filename (get_draft_filename ());
-
   GtkWidget * d = gtk_file_chooser_dialog_new (
     _("Save Draft Article"),
     GTK_WINDOW(_root),
@@ -1199,13 +1204,17 @@ PostUI :: save_draft ()
     NULL);
   gtk_dialog_set_default_response (GTK_DIALOG(d), GTK_RESPONSE_ACCEPT);
 
-  if (!gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(d), draft_filename.c_str()))
+  std::string& draft_filename (get_draft_filename ());
+  if (g_file_test (draft_filename.c_str(), G_FILE_TEST_IS_DIR))
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(d), draft_filename.c_str());
+  else
     gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(d), draft_filename.c_str());
 
   if (gtk_dialog_run(GTK_DIALOG(d)) == GTK_RESPONSE_ACCEPT)
   {
     GMimeMessage * msg = new_message_from_ui (DRAFTING);
     char * filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(d));
+    draft_filename = filename;
 
     errno = 0;
     std::ofstream o (filename);
@@ -1358,20 +1367,15 @@ namespace
       return;
 
     char * pch = g_strndup (v.str, v.len);
-    char * sig = 0;
+    std::string sig;
 
     if (type == Profile::TEXT)
     {
-      sig = g_strdup (pch);
+      sig = pch;
     }
     else if (type == Profile::FILE)
     {
-      GError * err = NULL;
-      if (!g_file_get_contents (pch, &sig, NULL, &err))
-      {
-        Log::add_err_va (_("Couldn't read signature file \"%s\": %s"), pch, err->message);
-        g_error_free (err);
-      }
+      file :: get_text_file_contents (pch, sig);
     }
     else // command
     {
@@ -1403,7 +1407,7 @@ namespace
         int exit_status = 0;
 
         if (g_spawn_sync (NULL, argv, NULL, GSpawnFlags(0), NULL, NULL, &spawn_stdout, &spawn_stderr, &exit_status, NULL))
-          sig = g_strdup (spawn_stdout);
+          sig = spawn_stdout;
         if (spawn_stderr && *spawn_stderr)
           Log::add_err (spawn_stderr);
 
@@ -1419,25 +1423,16 @@ namespace
      * If we can't convert, clear the signature. Otherwise, we'd add an
      * charset-encoded sig (say 'iso-8859-1') to the body (in UTF-8),
      * which could result in a blank message in the composer window. */
-    if (sig!=NULL)
-    {
-      const std::string s (content_to_utf8 (sig));
-      g_free (sig);
-      sig = g_strndup (s.c_str(), s.size());
-    }
+    if (!sig.empty())
+      sig = content_to_utf8 (sig);
     else
-    {
       Log::add_err (_("Couldn't convert signature to UTF-8."));
-      g_free (sig);
-      sig = NULL;
-    }
 
-    if (sig)
+    if (!sig.empty())
       setme = sig;
 
     /* cleanup */
     g_free (pch);
-    g_free (sig);
   }
 }
 
@@ -1537,6 +1532,8 @@ PostUI :: apply_profile_to_body ()
   StringView v (body.c_str(), index);
   v.rtrim ();
   body.assign (v.str, v.len);
+  if (!body.empty())
+    body += "\n\n"; // add linefeed to the last quoted line, and add a blank line between quoted and new text
   const int insert_pos = body.size();
 
   // insert the new signature
@@ -1559,7 +1556,7 @@ PostUI :: apply_profile_to_body ()
   gtk_text_buffer_move_mark_by_name (buf, "selection_bound", &iter);
   gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(_body_view),
                                 gtk_text_buffer_get_mark(buf, "insert"),
-                                0.0, false, 0.0, 0.0);
+                                0.0, true, 0.0, 0.5);
 }
 
 void
@@ -1596,6 +1593,13 @@ PostUI :: apply_profile_to_headers ()
   foreach_const (Profile::headers_t, profile.headers, it)
     headers[it->first] = it->second;
 
+  // if user has custom reply-to header, handle that here.
+  Profile::headers_t::iterator p = headers.find ("Reply-To");
+  if (p != headers.end()) {
+    gtk_entry_set_text (GTK_ENTRY(_replyto_entry), p->second.c_str());
+    headers.erase (p);
+  }
+
   // rewrite the extra headers pane
   std::string s;
   foreach_const (Profile::headers_t, headers, it)
@@ -1968,7 +1972,7 @@ PostUI :: create_extras_tab ()
 
   ++row;
   w = _message_id_check = gtk_check_button_new_with_mnemonic (_("Add \"Message-_Id header"));
-  b = _prefs.get_flag (MESSAGE_ID_PREFS_KEY, true);
+  b = _prefs.get_flag (MESSAGE_ID_PREFS_KEY, false);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), b);
   g_signal_connect (w, "toggled", G_CALLBACK(message_id_toggled_cb), &_prefs);
   gtk_table_attach (GTK_TABLE(t), w, 0, 2, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
