@@ -255,6 +255,8 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
                               Health             health,
                               const StringView & response)
 {
+  bool err_local = false;
+
   // find the Needed using this nntp...
   needed_t::iterator it;
   for (it=_needed.begin(); it!=_needed.end(); ++it)
@@ -266,14 +268,15 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
 
   if (health == OK) { // if download succeeded, save it in the cache
     const StringView view (&it->buf.front(), it->buf.size());
-    _cache.add (it->part.get_message_id(_article.message_id), view);
+    if (!_cache.add (it->part.get_message_id(_article.message_id), view))
+      err_local = true;
   }
 
-  if (health == COMMAND_FAILED) // if server doesn't have that article...
+  if (health == ERR_COMMAND) // if server doesn't have that article...
     it->xref.remove_server (nntp->_server);
 
   // if we got it or still have other options, we're okay
-  _state.set_health (health==COMMAND_FAILED && it->xref.empty() ? COMMAND_FAILED : OK);
+  _state.set_health (health==ERR_COMMAND && it->xref.empty() ? ERR_COMMAND : OK);
 
   if (health==OK)
     _needed.erase (it);
@@ -284,7 +287,7 @@ TaskArticle :: on_nntp_done  (NNTP             * nntp,
   }
 
   update_work ();
-  check_in (nntp, health);
+  check_in (nntp, err_local ? ERR_LOCAL : health);
 }
 
 /***
@@ -326,6 +329,8 @@ TaskArticle :: on_worker_done (bool cancelled)
     // the decoder is done... catch up on all housekeeping
     // now that we're back in the main thread.
 
+    foreach_const(std::list<std::string>, _decoder->log_severe, it)
+      Log :: add_err(it->c_str());
     foreach_const(std::list<std::string>, _decoder->log_errors, it)
       Log :: add_err(it->c_str());
     foreach_const(std::list<std::string>, _decoder->log_infos, it)
@@ -337,9 +342,13 @@ TaskArticle :: on_worker_done (bool cancelled)
     if (!_decoder->log_errors.empty())
       set_error (_decoder->log_errors.front());
 
-    _state.set_completed();
-    set_step (100);
-    _decoder_has_run = true;
+    if (!_decoder->log_severe.empty())
+      _state.set_health (ERR_LOCAL);
+    else {
+      _state.set_completed();
+      set_step (100);
+      _decoder_has_run = true;
+    }
   }
 
   Decoder * d (_decoder);
